%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%                                                                 %%%
%%%	MATLAB Code for:					       							     %%% 
%%%								       										     %%%
%%%			GENERATIONAL ACCOUNTING			       					     %%%
%%%			(with 1st, 2nd, and 3rd+ generation immigrants) 	     %%%
%%%								       										     %%%
%%%								       										     %%%
%%%	Programming by:  	Philip Oreopoulos		       				     %%%
%%%								       										     %%%
%%%	Filename:		gamatimm.m			       						     %%%
%%%								       										     %%%
%%%								       										     %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;

%==================================%
%  Parameter Settings and Switches %
%==================================%

% policy adjustment parameters:

switch04 = 0;				% policy adjustment for all gen. if 1;
startingyear = 1995;		% starting year of policy if currentgen=1
numtaxtypes = 0;			% total number of tax types to adjust
numtransfertypes = 8;	% total number of transfer types to adjust

taxcat = [9 10 11 14 15];		
transfercat = [1 2 3 4 5 6 7 8];
								% matrix of tax/transfer category numbers

% Codes are:
%						Taxes								Tramsfers
%
%					9  Labour Income Taxes			1  OASDI
%					10  FICA Taxes						2  Medicare		
%					11 Excise Taxes					3  Medicaid 
%					12 Cap. Income Taxes				4  UI
%					13 Inframarginal Cap. 			5  General Welfare
%					14 Monetary Base					6  AFDC
%					15 Property Taxes					7 Food Stamps
%															8 Eduction
%															16 Fed. Government Cons.
%															17 State Government Cons.


DEBT = 2078610000000;

switch01 = 0;				% omits cap. tax. adj. if '1'
switch02 = 0;				% also calculates m & f together
switch06 = 2;				% specifies aggregate projections
								% 1=low,2=med,3=high,4=with edu.
switch07 = 2;				% specifies demographic projections
								% (1=low, 2=med, 3=high)
switch08 = 0;				% 1= no immigration after 2000
switch09 = 0;				% treat nat. defense as pub good
								% for comparison of immigrants

                   
r=0.060;	
g=0.012;						% Note: also adjust growth rate below				
ages=101;     				
types=17;    				
			
baseyear 	= 1995;			
lastyear 	= 2100;			
projyear 	= 2070;				

											% these definitions make the notation
prjyear=projyear-baseyear+1;		% easier to follow when working
lstyear=lastyear-baseyear+1;		% with column and row numbers
remainyear=lastyear-projyear+1;




%=====================================%
% Load in data files and conform data %
%=====================================%

load POPM1.txt;, load POPM2.txt;, load POPM3.txt;	% loads pop. data
load POPF1.txt;, load POPF2.txt;, load POPF3.txt;
   
load POPM1Z.txt;, load POPM2Z.txt;, load POPM3Z.txt;	% loads pop. data
load POPF1Z.txt;, load POPF2Z.txt;, load POPF3Z.txt;

% force lower triangle after the year 2000 
% of 1st generation male and females
% to zeros (to get rid of interpolation problem)

POPM1Z = triu(POPM1Z,-5); POPF1Z = triu(POPF1Z,-5);

% force upper triangles of 3rd generation to be the same
% as base projections (native born pop the same)

POPM3Z = triu(POPM3,0) + tril(POPM3Z,-1);
POPF3Z = triu(POPF3,0) + tril(POPF3Z,-1);


POPM1(:,1)=[];, POPM2(:,1)=[];, POPM3(:,1)=[];		%removes year column
POPF1(:,1)=[];, POPF2(:,1)=[];, POPF3(:,1)=[];

POPM1Z(:,1)=[];, POPM2Z(:,1)=[];, POPM3Z(:,1)=[];		%removes year column
POPF1Z(:,1)=[];, POPF2Z(:,1)=[];, POPF3Z(:,1)=[];


load RPM1.txt;, load RPM2.txt;, load RPM3.txt;		%load rel. prf. data
load RPF1.txt;, load RPF2.txt;, load RPF3.txt;

RPM1(:,1:2)=[];, RPM2(:,1:2)=[];, RPM3(:,1:2)=[];	%removes year and type
RPF1(:,1:2)=[];, RPF2(:,1:2)=[];, RPF3(:,1:2)=[];	%column

  

if switch06 == 1
   load AGGLO.txt; AGG=AGGLO;clear AGGLO;g=0.007;
elseif switch06 == 2
   load AGGMED.txt; AGG=AGGMED;clear AGGMED;g=0.012;
elseif switch06 == 3
   load AGGHI.txt; AGG=AGGHI;clear AGGHI;g=0.017;
elseif switch06 == 4
   load AGGEDUC.txt; AGG=AGGEDUC;clear AGGEDUC;g=0.012;
end

				      	% Aggregate matrix has dimensions
AGG(:,1)=[];			% YEAR x TYPE. first clumn
							% displaying the year is removed
							% NOTE: the last two columns contain
							% the values of federal and state government
							% consumption
 


%---------------------------------------------------------------%
% Gets discount and growth matricies with dims AGE x YEAR, from %
% baseyear to lastyear: (1+g)^0, (1+g)^1, (1+g)^2, ...          %
%---------------------------------------------------------------%

for i = 1:lstyear,DSCT(:,i) = (1/(1+r)^(i-1))*ones(ages,1);,end


for j = 1:lstyear,GRTH(:,j) = (1+g)^(j-1)*ones(ages,1);,end

%==================================================%
% Converts Aggregates to billions of U.S. dollars  %
% and defines some variables			   				%
%==================================================%

AGG = 1000000000*AGG;

DEBT = 2078610000000;

%DEBT =   1000000000*DEBT; 

%===============================================%
%															   %
% Capital Income Tax Adjustment (If switch01=0) %
%															   %			
%===============================================%

qcap 	= 0.11100;
delcap 	= 0.00111;

if switch01 == 1, AGG(:,13) = 0;, end;

for i = 1:prjyear;
   AGG(i,12) = AGG(i,12) - (delcap*AGG(1,13))*((1+g)^(i-1));
end;

AGG(:,13) = qcap*AGG(:,13);


%==================================================================%
%								   													 %
% Calculates from data per capita amounts of taxes paid 	   		 %
% and transfers received for each age group by year out to 2200    %
% 								   													 %
%        PERCAP matrix is (age,year,type,sex,immgen)		   		 %
%								   													 %
%==================================================================%

SUMS = RPM1*POPM1(1:prjyear,:)' + RPF1*POPF1(1:prjyear,:)' + ...
       RPM2*POPM2(1:prjyear,:)' + RPF2*POPF2(1:prjyear,:)' + ...
       RPM3*POPM3(1:prjyear,:)' + RPF3*POPF3(1:prjyear,:)'; 

UNITIES = AGG./SUMS';


PERCAP(:,:,:,1,1) = repmat(permute(RPM1,[2,3,1]),[1,prjyear,1]) .* ...
	   	    repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);
PERCAP(:,:,:,2,1) = repmat(permute(RPF1,[2,3,1]),[1,prjyear,1]) .* ...
	            repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);
PERCAP(:,:,:,1,2) = repmat(permute(RPM2,[2,3,1]),[1,prjyear,1]) .* ...
	   	    repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);
PERCAP(:,:,:,2,2) = repmat(permute(RPF2,[2,3,1]),[1,prjyear,1]) .* ...
	   	    repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);
PERCAP(:,:,:,1,3) = repmat(permute(RPM3,[2,3,1]),[1,prjyear,1]) .* ...
	   	    repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);
PERCAP(:,:,:,2,3) = repmat(permute(RPF3,[2,3,1]),[1,prjyear,1]) .* ...
	   	    repmat(permute(UNITIES,[3,1,2]),[ages,1,1]);


% extend percap matrix from 2070 to 2094

PERCAP(:,prjyear+1:ages+1,:,:,:) = ...
	repmat(PERCAP(:,prjyear,:,:,:),[1,ages-prjyear+1,1,1,1]) .* ...
	repmat(GRTH(:,2:ages-prjyear+2),[1,1,types,2,3]);

 
clear SUMS UNITIES;


%==================================================%
%						   										%
% Aggregates PERCAP matrix and converts to PV      %
% PVTOTALS (age,year,type,sex,immgen               %
%															      %
% Uses no immigration projections if switch08=1    %
%						   										%
%==================================================%

% collects population data into one matrix

if switch08==0;

POP(:,:,1,1,1) = POPM1';
POP(:,:,1,2,1) = POPF1'; 
POP(:,:,1,1,2) = POPM2'; 
POP(:,:,1,2,2) = POPF2'; 
POP(:,:,1,1,3) = POPM3'; 
POP(:,:,1,2,3) = POPF3';

else
   
POP(:,:,1,1,1) = POPM1Z';
POP(:,:,1,2,1) = POPF1Z';
POP(:,:,1,1,2) = POPM2Z';
POP(:,:,1,2,2) = POPF2Z';
POP(:,:,1,1,3) = POPM3Z';
POP(:,:,1,2,3) = POPF3Z';

end;


TOTALPOP = repmat(POP,[1,1,types,1,1]);


PVTOTALS = PERCAP .* repmat(DSCT(:,1:ages+1),[1,1,types,2,3]) .* ...
   		     TOTALPOP(:,1:ages+1,:,:,:);

if switch09==1
   publicgood;
end;


%==================================================%
%	                                 	   			%
% gets generational accounts for living age groups %
%						   										%
% BEFORE policy adjustment			   					%
%						   										%
%==================================================%

for type = 1:types, for age = 1:ages, for sex = 1:2, for immgen = 1:3 
   
TAXTRANSBEFORE(age,type,sex,immgen) = ...
 	sum(diag(PVTOTALS(:,1:ages,type,sex,immgen),-(age-1))) ./ ...
				POP(age,1,1,sex,immgen);

end, end, end, end


for sex = 1:2, for immgen = 1:3
	GABEFORE(:,sex,immgen) = sum(TAXTRANSBEFORE(:,1:15,sex,immgen),2);	
end,end


GAFUTUREB = GABEFORE(1,:,:)*(1+g);

%==================================================%
%	                                 	   			%
% calculates males and females together if		   %
%						   										%
% switch02=1							   					%
%						   										%
%==================================================%

if switch02 == 1
   MFTOTALS=sum(PVTOTALS,4);
   MFPOP=sum(POP(:,1:2,1,:,:),4);
   
   for type = 1:types, for age = 1:ages, for immgen = 1:3    
		TAXTRANSBEFORE(age,type,3,immgen) = ...
 		sum(diag(MFTOTALS(:,1:ages,type,1,immgen),-(age-1))) ./ ...
													MFPOP(age,1,1,1,immgen);
	end, end, end

	GABEFORE(:,3,:) = sum(TAXTRANSBEFORE(:,1:15,3,:),2);	
end


%==============================================================%
%							       												%
% calculate gap if no policy change                            %
%							       												%
%==============================================================%

% The gap is the debt -
% the sum of all PV net payments for living generations -
% the PV net payments for future generations up to 2200 -
% the PV net payments for future generations extended out to infinity



netpvtaxes = sum(sum(sum(sum(sum(PVTOTALS(:,1:ages,:,:,:)))))) + 	...
								 														...
								 														...
	     sum(sum(sum(sum(sum(                           	 			...
	repmat(PERCAP(:,ages+1,:,:,:),[1,lstyear-ages,1,1,1]) .* 		...
	repmat(GRTH(:,1:lstyear-ages),[1,1,types,2,3]) .*	 	 			...
	repmat(DSCT(:,ages+1:lstyear),[1,1,types,2,3])   .* 	 			...
   repmat(TOTALPOP(:,ages+1:lstyear,:,:,:),[1,1,1,1,1])))))) 		...
								 														...
				+				 														...
								 														...
	     sum(sum(sum(sum(sum(                           	 			...
	PERCAP(:,ages+1,:,:,:) .* 	 			 									...
	repmat(GRTH(:,lstyear-ages),[1,1,types,2,3]) .* 	 				...
	repmat(DSCT(:,lstyear),[1,1,types,2,3])   .* 	         		...
        repmat(TOTALPOP(:,lstyear,:,:,:),[1,1,1,1,1])))))) *      	...
	(1+g)*(1/(1-((1+g)/(1+r))));	       


govgap = DEBT - netpvtaxes;


%==============================================================%
%							       												%
% estimate percentage increase/decrease of tax/transfer types  %
% specified to eliminate gap.	                               	%
%							       												%
%	calculated for FUTURE generations only		       				%
%							       												%
%==============================================================%

% the percentage increase in particular tax/transfer programs is
% caclulated by summing all the PV payments/receipts of these
% programs for future generations, out to infinity, and calculating
% how much increase is required to remove the gap

% we calculate for tax and transfers seperately

% NOTE: the 'triu,1' command keeps the upper triangular portion of the 
% PERCAP matrix, shifted to the right one, which is the PV TOTALS for 
% future generations between 1996 and 2096.  The rest of the matrix 
% (for the percap amounts for living generations) become zero


for j=1:numtaxtypes; 

taxtotal(j) = 																				...
																								...	
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),1,1),1))))) + 			...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),2,1),1))))) + 			...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),1,2),1))))) + 			...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),2,2),1))))) + 			...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),1,3),1))))) + 			...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,taxcat(j),2,3),1))))) + 			...
																								...
								 																...
	     sum(sum(sum(sum(sum(     	                      	 				...
repmat(PERCAP(:,ages+1,taxcat(j),:,:),[1,lstyear-ages,1,1,1]).*		   ...
repmat(GRTH(:,1:lstyear-ages),[1,1,1,2,3]) .* 	 								...	
repmat(DSCT(:,ages+1:lstyear),[1,1,1,2,3])   .* 								...
repmat(POP(:,ages+1:lstyear,:,:,:),[1,1,1,1,1]))))))	 						...
								 																...
				+				 																...
								 																...
	     sum(sum(sum(sum(sum(                           	 					...
	PERCAP(:,ages+1,taxcat(j),:,:) .* 	 												...
	repmat(GRTH(:,lstyear-ages),[1,1,1,2,3]) .* 								...
	repmat(DSCT(:,lstyear),[1,1,1,2,3])   .* 	        							...
        repmat(POP(:,lstyear,:,:,:),[1,1,1,1,1])))))) *       				...
	(1+g)*(1/(1-((1+g)/(1+r))));	       

end


if numtaxtypes == 0, taxtotal=[0];,end;

for k=1:numtransfertypes; 

transfertotal(k) =  -																	...
																								...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),1,1),1))))) - 		...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),2,1),1))))) - 		...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),1,2),1))))) - 		...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),2,2),1))))) - 		...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),1,3),1))))) - 		...
sum(sum(sum(sum(triu(PVTOTALS(:,1:ages,transfercat(k),2,3),1))))) - 		...
																								...
																								...
	     sum(sum(sum(sum(sum(     	                      	 				...
repmat(PERCAP(:,ages+1,transfercat(k),:,:),[1,lstyear-ages,1,1,1]).*		...
repmat(GRTH(:,1:lstyear-ages),[1,1,1,2,3]) .* 	 								...	
repmat(DSCT(:,ages+1:lstyear),[1,1,1,2,3])   .* 								...
repmat(POP(:,ages+1:lstyear,:,:,:),[1,1,1,1,1])))))) 							...
								 																...
				-				 																...
								 																...
	     sum(sum(sum(sum(sum(                           	 					...
	PERCAP(:,ages+1,transfercat(k),:,:) .* 	 									...
	repmat(GRTH(:,lstyear-ages),[1,1,1,2,3]) .* 									...
	repmat(DSCT(:,lstyear),[1,1,1,2,3])   .* 										...
        repmat(POP(:,lstyear,:,:,:),[1,1,1,1,1])))))) *  					...
	(1+g)*(1/(1-((1+g)/(1+r))));	       

end


if numtransfertypes == 0, transfertotal=[0];,end;



			% NOTE: we subtract to make the aggregate positive

percentagechange = govgap / (sum(taxtotal) + sum(transfertotal)) 



%==============================================================%
%							       												%
% policy adjustment for living generations if switch04=1       %
%							       												%
%==============================================================%

% simply adds on the additional PV net tax payments for living generations
% and then calculates the percentage change again

if switch04 ==1,

for j=1:numtaxtypes;
taxtotal(j) = 	taxtotal(j) + ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),1,1)))))) + ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),2,1)))))) + ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),1,2)))))) + ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),2,2)))))) + ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),1,3)))))) + ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,taxcat(j),2,3))))));
end

if numtaxtypes == 0, taxtotal=[0];,end;

for k=1:numtransfertypes;
transfertotal(k) =  transfertotal(k) - ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),1,1)))))) - ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),2,1)))))) - ...
   sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),1,2)))))) - ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),2,2)))))) - ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),1,3)))))) - ...
	sum(sum(sum(sum(tril(PVTOTALS(:,1:ages,transfercat(k),2,3))))));
end
 
if numtransfertypes == 0, transfertotal=[0];,end;


percentagechange = govgap / (sum(taxtotal) + sum(transfertotal)) 



end;



%==============================================================%
%							       												%
% computes new generational accounts, after policy change      %
% for future generations affected only...		       				%
%							       												%
%==============================================================%


if switch04==0,

TAXTRANSAFTER = TAXTRANSBEFORE;   
   
GAAFTER = GABEFORE;

for j=1:numtaxtypes; for sex = 1:2; for immgen = 1:3;  
	PERCAP(:,1:ages+1,taxcat(j),sex, immgen) = ...
	tril(PERCAP(:,1:ages+1,taxcat(j),sex,immgen)) + ...
	(triu(PERCAP(:,1:ages+1,taxcat(j),sex,immgen),1)*(1+percentagechange));
end, end, end;

for k=1:numtransfertypes; for sex = 1:2; for immgen = 1:3;
	PERCAP(:,:,transfercat(k),sex,immgen) = ...
	tril(PERCAP(:,:,transfercat(k),sex,immgen)) + ...
	triu(PERCAP(:,:,transfercat(k),sex,immgen),1)*(1-percentagechange);
end, end, end;


PVTOTALSF = PERCAP(:,2:102,:,:,:) .* ...
            repmat(DSCT(:,1:ages),[1,1,types,2,3]) .* ...
   		   repmat(POP(:,2:ages+1,:,:,:),[1,1,types,1,1]);

for type=1:types;, for sex = 1:2;, for immgen=1:3;
FutureTaxTrans(type,sex,immgen) = ...
	 sum(diag(PVTOTALSF(:,:,type,sex,immgen),0)) ./ ...
					POP(1,2,1,sex,immgen);
end,end,end



GAFUTUREA = sum(FutureTaxTrans(1:15,:,:))/(1+g);


%==============================================%
% combines for males and females if switch02=1 %
%==============================================%

if switch02 == 1
   MFTOTALS=sum(PVTOTALS,4);
   
   for type = 1:types, for immgen = 1:3    
		FutureTaxTrans(type,3,immgen) = ...
 		sum(diag(MFTOTALS(:,:,type,1,immgen),1)) ./ ...
													MFPOP(1,2,1,1,immgen);
	end, end

GAFUTUREA(:,3,:)=sum(FutureTaxTrans(1:15,3,:))/(1+g);
end


%================================================%
%						 										 %
% Computes new GAs, with policy adjusted			 %
% for everyone if switch04=1  				       %
%						 										 %
%================================================%

elseif switch04==1


for j=1:numtaxtypes;
PERCAP(:,:,taxcat(j),:,:) = PERCAP(:,:,taxcat(j),:,:)*(1+percentagechange);
end

for k=1:numtransfertypes;
PERCAP(:,:,transfercat(k),:,:) = ...
		PERCAP(:,:,transfercat(k),:,:) * (1-percentagechange);
end

PVTOTALS = PERCAP .* repmat(DSCT(:,1:ages+1),[1,1,types,2,3]) .* ...
   		     repmat(TOTALPOP(:,1:ages+1,:,:,:),[1,1,1,1,1]);


for type = 1:types, for age = 1:ages, for sex = 1:2, for immgen = 1:3
   TAXTRANSAFTER(age,type,sex,immgen) = ...
	sum(diag(PVTOTALS(:,1:ages,type,sex,immgen),-(age-1))) ./ ...
						TOTALPOP(age,1,1,sex,immgen);
end, end, end, end

for sex = 1:2, for immgen = 1:3
	GAAFTER(:,sex,immgen) = sum(TAXTRANSAFTER(:,1:15,sex,immgen),2);	
end,end



% same now for future generations (the cohort born after
% the base year for comparison purposes


PVTOTALSF = PERCAP(:,2:102,:,:,:) .* ...
            repmat(DSCT(:,1:ages),[1,1,types,2,3]) .* ...
   		   repmat(POP(:,2:ages+1,:,:,:),[1,1,types,1,1]);

for type=1:types;, for sex = 1:2;, for immgen=1:3;
FutureTaxTrans(type,sex,immgen) = ...
	 sum(diag(PVTOTALSF(:,:,type,sex,immgen),0)) ./ ...
					POP(1,2,1,sex,immgen);
end,end,end



GAFUTUREA = sum(FutureTaxTrans(1:15,:,:))/(1+g);




%================================================================%
% calculates GAs with males and females together if switch02 = 1 %
%================================================================%

if switch02 == 1
   MFTOTALS=sum(PVTOTALS,4);
   
   for type = 1:types, for age = 1:ages, for immgen = 1:3    
		TAXTRANSAFTER(age,type,3,immgen) = ...
 		sum(diag(MFTOTALS(:,1:ages,type,1,immgen),-(age-1))) ./ ...
													MFPOP(age,1,1,1,immgen);
	end, end, end

	GAAFTER(:,3,:) = sum(TAXTRANSBEFORE(:,1:15,3,:),2);	
   GAFUTUREA(:,3,:)=sum(FutureTaxTrans(1:15,3,:))/(1+g);
end


end





